@@ -266,3 +266,39 @@ $VERSION = '1.01';
 );
 
 1;  # end
+
+__END__
+
+=head1 NAME
+
+Image::ExifTool::Casio - Definitions for Casio EXIF maker notes
+
+=head1 SYNOPSIS
+
+This module is loaded automatically by Image::ExifTool when required.
+
+=head1 DESCRIPTION
+
+This module contains definitions required by Image::ExifTool to interpret
+Casio maker notes in EXIF information.
+
+=head1 AUTHOR
+
+Copyright 2003-2004, Phil Harvey (phil at owl.phy.queensu.ca)
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself.
+
+=head1 REFERENCES
+
+=over 4
+
+=item http://park2.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html
+
+=back
+
+=head1 SEE ALSO
+
+L<Image::ExifTool|Image::ExifTool>
+
+=cut
